topGOres2 <- function (ids, ontology = "BP", Pthr = 1e-05, maxN = 5000, minN = 5, orgdb, allEG = keys(orgdb), showTerms = NULL) 
{
    if (!is.list(ids) & !is.character(ids))
        stop("ids has to be of class character or list ...")
    if (is.list(ids) & !is.character(unlist(ids))) 
        stop("elements of ids has to be of class character ...")
    if (!all(ontology %in% c("BP", "CC", "MF"))) 
        stop("ontology has to be one of: CC, BP, MF ...")
    if (!is.numeric(Pthr)) 
        stop("Pthr has to be of class character ...")
    if (!is.numeric(maxN)) 
        stop("maxN has to be of class character ...")
    if (!is.numeric(minN)) 
        stop("minN has to be of class character ...")
    if (!is(orgdb, "OrgDb")) 
        stop("orgdb has to be of class OrgDb ...")
    if (species(orgdb) != "Homo sapiens" && species(orgdb) != 
        "Mus musculus" && species(orgdb) != "Drosophila melanogaster") 
        stop("only org.Hs.eg.db, org.Mm.eg.db and org.Dm.eg.db are supported ...")
    if (!is(allEG, "character")) 
        stop("allEG has to be of class character ...")
    if (!is.null(showTerms) && !is.numeric(showTerms)) 
        stop("showTerms has to be either NULL or of class numeric ...")
    if (!is.list(ids))
        ids <- list(ids)
    EGvec <- unique(c(allEG, unlist(ids)))
    EGvec <- rep(0, length(allEG))
    names(EGvec) <- allEG
    EGvec[ids[[1]]] <- 1
    EGvec <- factor(EGvec)
    if (length(levels(EGvec)) == 1) 
        stop("the allEG gene universe cannot be the same as the provided ids ...")
    if (species(orgdb) == "Homo sapiens") 
        dbname <- "org.Hs.eg.db"
    if (species(orgdb) == "Mus musculus") 
        dbname <- "org.Mm.eg.db"
    if (species(orgdb) == "Drosophila melanogaster") 
        dbname <- "org.Dm.eg.db"
    GOdata <- new("topGOdata", ontology = ontology, allGenes = EGvec, 
        annot = annFUN.org, mapping = dbname, nodeSize = minN)
    topGOlist <- tryCatch(lapply(seq_along(ids), function(i) {
        
        EGvec <- rep(0, length(allEG))
        names(EGvec) <- allEG
        EGvec[ids[[i]]] <- 1
        EGvec <- factor(EGvec)
        GOdata <- updateGenes(GOdata, EGvec)

        res <- runTest(GOdata, algorithm = "classic", statistic = "fisher")
        resdf <- GenTable(GOdata, classic = res, topNodes = 50)
        ann.genes <- genesInTerm(GOdata, whichGO = resdf$GO.ID)
        Genes <- sapply(ann.genes, function(x) {
            intersect(x, ids[[i]])
        })
        Genes <- sapply(Genes, function(x) {
            c(paste(x, sep = "", collapse = "/"))
        })
        Genes <- as.data.frame(Genes)
        resdf <- cbind(resdf, Genes)
        resdf$Term <- Term(GOTERM[resdf$GO.ID])
        pvals <- resdf$classic
        pvals <- as.numeric(sub("<", "", pvals))
        resdf$classic <- pvals
        inds <- which(resdf$Annotated < maxN & pvals < Pthr)
        if (length(inds) == 0) 
            return(NULL)
        resdf <- resdf[inds, ]
        if (is.numeric(showTerms)) {
            if (showTerms > 50) 
                showTerms <- 50
            if (showTerms > nrow(resdf)) 
                showTerms <- nrow(resdf)
            enrich_GO <- resdf[1:showTerms, ]
            enrich_GO$Term <- factor(enrich_GO$Term, levels = enrich_GO$Term)
            enrich_GO$P_val <- -log10(as.numeric(enrich_GO$classic))
            g.plot <- ggplot(enrich_GO, aes(Term, Significant, fill = P_val)) + 
                geom_bar(stat = "identity") + coord_flip() + ylab("Number of Genes")
            return(list(data = resdf, enrichplot = g.plot))
        }
        return(resdf)
    }), error=function(e) e)

    if (length(topGOlist)==1) {
        return(topGOlist[[1]]) 
    } else {
        return(topGOlist)
    }
}

# Functions to summarize the results of the functional analysis
enrichmentsSumUp <- function(annotations, th=-3)
{

    lapply(annotations[c(1:4,6,9:11)],function(a)
    {
        boolTmp <- a$Hyper_Raw_PValue <= th & a$Binom_Raw_PValue <= th 
        cbind(name = a$name[boolTmp]
            , hyper = format(a$Hyper_Raw_PValue[boolTmp],scientific=TRUE,digits=1)
            , binom = format(a$Binom_Raw_PValue[boolTmp],scientific=TRUE,digits=1))


    })

}

collapsedGOterms <- function(a,b)
{
    if(is.null(dim(a)))
    {
        a_tmp <- a[[1]]
        for(i in a[-1])
        {
            a_tmp <- rbind(a_tmp,i) 
        }
        a <- a_tmp
    
        b_tmp <- b[[1]]
        for(i in b[-1])
        {
            b_tmp <- rbind(b_tmp,i) 
        }
        b <- b_tmp
        rownames(a) <- a[,1]
        rownames(b) <- b[,1]
    }

    terms_tmp <- intersect(rownames(a),rownames(b))
    b <- b[,c("hyper","binom")]
    b <- b[terms_tmp,]

    rownames(b) <- substr(rownames(b),1,45)

    ord_tmp <- sort.int(as.numeric(b[,1])+as.numeric(b[,2]),index.return=TRUE)

    return(b[ord_tmp$ix,])

}

differentialGOterms <- function(a,c)
{
    terms_tmp <- setdiff(rownames(a),rownames(c))
    a[terms_tmp,]
}

commonGOterms <- function(a,b)
{
    terms_tmp <- intersect(rownames(a),rownames(b))
    cbind(a[terms_tmp,],b[terms_tmp,])
}

# Functions to insert white columns and rows in the heatmap for graphical purposes
colInsert <- function(matrix,position)
{
    cbind(matrix[,1:(position-1)],rep(NaN,nrow(matrix)),matrix[,c(position):ncol(matrix)])
}

colsInsert <- function(matrix,positions)
{
    positions <- sort(positions,decreasing=FALSE)
    positions <- positions+(seq_along(positions)-1)
    for(i in positions)
    {
        matrix <- colInsert(matrix,i)
    }
    matrix
}

rowInsert <- function(mat,position)
{
    rbind(mat[1:(position-1),],matrix(NaN,nrow=100,ncol=ncol(mat)),mat[c(position):nrow(mat),])
}

rowsInsert <- function(mat,positions)
{
    positions <- sort(positions,decreasing=FALSE)
    positions <- positions+(seq_along(positions)-1)*100
    for(i in positions)
    {
        mat <- rowInsert(mat,i)
    }
    mat
}

# Function to infer not finite rates
impute_na_tc <- function(tpts, tcdata) {

    # impute NA values in a time course data as a linear model between non-NA values
    tc_impute_NA_linearmodel <- function(tpts, tcdata) {
        for( j in seq_along(tcdata) ) {
            if( is.na(tcdata[j]) ) {
                lower_boundary_j <- j-1
                higher_boundary_j <- j+1
                while( is.na(tcdata[higher_boundary_j]) & higher_boundary_j <= length(tcdata) ) {
                    higher_boundary_j <- higher_boundary_j + 1
                }
                if( lower_boundary_j > 0 & higher_boundary_j <= length(tcdata) ) 
                    if ( is.finite(tcdata[lower_boundary_j]) ) {
                        x <- tpts[c(lower_boundary_j,higher_boundary_j)]
                        y <- tcdata[c(lower_boundary_j,higher_boundary_j)]
                        tcdata[(lower_boundary_j+1):(higher_boundary_j-1)] <- predict(lm(y ~ x), 
                            data.frame(x=tpts[(lower_boundary_j+1):(higher_boundary_j-1)]))
                    }
            }
        }
        return(tcdata)
    }

    # impute NA values in a time course from boundary values
    tc_impute_NA_boundaries <- function(tcdata) {

        forward_direction <- function(tcdata) {
            if( is.na(tcdata[1]) & !all(is.na(tcdata)) ) {
                lower_boundary_j <- higher_boundary_j <- 1
                while( is.na(tcdata[higher_boundary_j] & higher_boundary_j < length(tcdata) ) ) {
                    higher_boundary_j <- higher_boundary_j + 1
                }
                tcdata[lower_boundary_j:(higher_boundary_j-1)] <- tcdata[higher_boundary_j]
            }
            return(tcdata)
        }

        tcdata <- forward_direction(tcdata)
        tcdata <- rev(forward_direction(rev(tcdata)))
        return(tcdata)
    }

    tcdata <- tc_impute_NA_linearmodel(tpts, tcdata)
    tcdata <- tc_impute_NA_boundaries(tcdata)
    return(tcdata)

}
